#include "mcc_generated_files/mcc.h"

#define TIME_OFF -1       //タイマーを使用しない場合
#define TIME_UP 0         //タイムアップ
#define TIME_FILTER_MAX 1 //ベースタイマカウント値
#define TIME_TX_MAX 10
#define	CNT_INIT_MAX 10    //10ms×10 = 100ms
#define RING_SZ 64
#define SZ_MASK 0x3F
#define DI_NUM 4           //DIフィルタのサンプリング数
#define TX_SZ 6

typedef struct{
    uint8_t wp;
    uint8_t rp;
    uint8_t buf[RING_SZ];
}RING_MNG;

typedef struct{
    uint8_t    wp;
    uint8_t    buf[ DI_NUM ];
    uint8_t    di;
}FILT_DATA;

typedef struct{
    uint8_t cnt;
    uint8_t data[TX_SZ];
}TX_MNG;

//--------------変数定義----------------------------
int8_t  timfilter;
int8_t  timTxWait; 
RING_MNG picRcv;
uint8_t CntInit; //初期化時のみ使用
int8_t rxData;
FILT_DATA DiData;
bool btn1hold;
TX_MNG  Txdata;
//--------------関数ﾌﾟﾛﾄﾀｲﾌﾟ宣言----------------------------
void mainApp(void);
void mainTimer(void); //ﾀｲﾏｰ0割り込みでの処理
void DiFilter(void);
void DiFilterInit(void);
void DiFilter(void);
void TxDataSet(void);
/* Main application */
void main(void)
{
    SYSTEM_Initialize();
    TMR0_SetInterruptHandler(mainTimer); //コールバック関数を指定
    INTERRUPT_GlobalInterruptEnable(); //割り込みの許可
    INTERRUPT_PeripheralInterruptEnable(); //ペリフェラル割り込みの許可
    DiFilterInit(); //メイン処理前にDIを確定する

    while (1)
    {
        mainApp();
        DiFilter();
    }
}
/* main処理  */
void mainApp(void){
    
    if(EUSART_is_rx_ready()){ //受信したデータがあるか
        rxData = EUSART_Read(); //受信データを取得
        picRcv.buf[picRcv.wp] = rxData;

        if( ++picRcv.wp >= sizeof(picRcv.buf)){
            picRcv.wp = 0;
        }
            
        if(EUSART_is_tx_ready()){ //送信可能であるか
            EUSART_Write(rxData); //データを送信
        }
        IO_RB6_Toggle();
    }
    
    if( DiData.di == 0 ){
        if( btn1hold == false ){
            btn1hold = true;
            TxDataSet(); //送信準備
        }
    }else{
        btn1hold = false; 
    }
    
    if( timTxWait > TIME_UP ){
        if(EUSART_is_tx_ready()){
            if( Txdata.cnt < sizeof(Txdata.data)){ //送信データ数に達したか
                EUSART_Write(Txdata.data[Txdata.cnt]); //送信
                ++Txdata.cnt; //次の送信データを選択
            }
            else{ //送信完了
                timTxWait = TIME_OFF;
            }
            IO_RA7_Toggle();
        }     
    }
    else if( timTxWait == TIME_UP){ //送信タイムアウト
        timTxWait = TIME_OFF;
    }   
}
/* 送信準備関数 */
void TxDataSet(void){
    
    Txdata.cnt = 0;
    Txdata.data[0] = 'P';
    Txdata.data[1] = 'I';
    Txdata.data[2] = 'C';
    Txdata.data[3] = '-';
    Txdata.data[4] = 'O';
    Txdata.data[5] = 'K';
    timTxWait = TIME_TX_MAX; //送信のタイムアウトをセット
}
/* タイマ管理関数（コールバック） */
void mainTimer(void){
 
    if( timfilter > TIME_UP ){
        --timfilter; //タイマを更新
    }
    
    if( timTxWait > TIME_UP){
        --timTxWait; 
    }
}
// DIフィルタの初期化 //
void DiFilterInit(void){
	
    CntInit = CNT_INIT_MAX;
    while(CntInit > 0){  //0になるまでフィルタを実施
	DiFilter();         //DIフィルタ処理
	__delay_ms(10);     //10ms遅延させてDIフィルタ処理
        CntInit--;
	timfilter = TIME_UP;
    }    
}
//--------モード切替部のDIフィルタ----------------------------
void DiFilter(void){
    uint8_t i;
    bool boo = true;

    if( timfilter == TIME_UP ){
        timfilter = TIME_FILTER_MAX;

        DiData.buf[ DiData.wp ] = IO_RB0_GetValue();

        for( i = 1; i < DI_NUM; i++){
            if( DiData.buf[i-1] != DiData.buf[i] ){
                boo = false;
                break;
            }
        }

        if( boo ){ //比較して一致したら値を採用
            DiData.di = DiData.buf[ 0 ]; 
            IO_RA3_LAT = DiData.di;
        }

        if( ++DiData.wp >= DI_NUM){
            DiData.wp = 0;
        }
    }
}